{*****************************************************************************}
{                                                                             }
{    GDI + Controls                                                           }
{      http://lummie.co.uk                                                    }
{        Version: 1.0.3                                                       }
{                                                                             }
{    Copyright (c) 2005 Matt harrison (http://www.lummie.co.uk)               }
{                                                                             }
{*****************************************************************************}
unit GDIPDropDownColorSelector;

interface

uses
  SysUtils, Classes, Controls, StdCtrls, TntStdCtrls, TntExDropDownEdit, ColorSelectorFrame,
  GDIPAPI, GDIPOBJ, GDIPUTIL, windows, messages;

type
  TColorChangeEvent = procedure (Sender : TObject; AlphaColor : TGPColor) of object;

  TGDIPDropDownColorSelector = class(TCustomTntExDropDownEdit)
  private
    { Private declarations }
    FColorSelector : TGDIPColorSelector;
    FCheckImage : TGPImage;
    FOnChange: TColorChangeEvent;
    procedure ColorChange(Sender : TObject; AlphaColor : TGPColor);
    function CreateCheckImage: TGPBitmap;
    function GetAllowAlpha: boolean;
    function GetAlphaColor: TGPColor;
    procedure SetAllowAlpha(const Value: boolean);
    procedure SetAlphaColor(const Value: TGPColor);
  protected
    { Protected declarations }
    function DoPopulatePanel(APanel: TTntExDropDownPanel): Boolean;
      override;
    procedure DoCustomPaint; override;
    procedure UpdateMargins; override;
    procedure DoDroppedDown; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property AlphaColor : TGPColor read GetAlphaColor write SetAlphaColor;
    property AllowAlpha : boolean read GetAllowAlpha write SetAllowAlpha;
    property OnChange : TColorChangeEvent read FOnChange write FOnChange;
  end;

implementation

{ TGDIPDropDownColorSelector }

uses
  GDIPExtensions;

const
  ColorSwashWidth  = 16;

procedure TGDIPDropDownColorSelector.ColorChange(Sender: TObject;
  AlphaColor: TGPColor);
begin
  Text := ColorAsHTML(AlphaColor, AllowAlpha);
  if assigned(FOnChange) then FOnChange(Self, AlphaColor);
end;

constructor TGDIPDropDownColorSelector.Create(AOwner: TComponent);
begin
  inherited;
  FOnChange := nil; 
  DropDownStyle := csDropDownList;
  FCheckImage := CreateCheckImage;
  PanelAutoWidth := false;
  PanelAllowResize := false;
  FColorSelector :=  TGDIPColorSelector.create(self);
  FColorSelector.OnChange := ColorChange;
end;

function TGDIPDropDownColorSelector.CreateCheckImage : TGPBitmap;
var
  ACanvas : TGPGraphics;
  ABrush : TGPSolidBrush;
begin
  try
    result := TGPBitmap.Create(15,15,PixelFormat24bppRGB);
    ACanvas := TGPGraphics.Create(result);
    ABrush := TGPSolidBrush.Create(MakeColor(255,255,255));
    ACanvas.FillRectangle(ABrush,MakeRect(0,0,15,15));
    ABrush.free;
    ABrush := TGPSolidBrush.Create(MakeColor(128,128,128));
    ACanvas.FillRectangle(ABrush,MakeRect(0,0,7,7));
    ACanvas.FillRectangle(ABrush,MakeRect(8,8,15,15));
    ABrush.free;
    ACanvas.Free;
  except
    result := nil;
  end;
end;


destructor TGDIPDropDownColorSelector.Destroy;
begin
  FreeAndNil(FCheckImage);
  inherited;
end;

procedure TGDIPDropDownColorSelector.DoCustomPaint;
var
  ABrush : TGPSolidBrush;
  APen : TGPPen;
  ACanvas, GPCanvas : TGPGraphics;
  ARect : TGPRect;
  AnImage : TGPBitmap;
begin
  ARect := MakeRect(2,2,ColorSwashWidth,height-8);

  AnImage := TGPBitmap.Create(15,15,PixelFormat24bppRGB);

  ACanvas := TGPGraphics.Create(AnImage);
    ABrush := TGPSolidBrush.Create(MakeColor(255,255,255));
    ACanvas.FillRectangle(ABrush,MakeRect(0,0,15,15));
    ABrush.free;
    ABrush := TGPSolidBrush.Create(MakeColor(128,128,128));
    ACanvas.FillRectangle(ABrush,MakeRect(0,0,7,7));
    ACanvas.FillRectangle(ABrush,MakeRect(8,8,15,15));
    ABrush.free;
    ABrush := TGPSolidBrush.create(FColorSelector.AlphaColor) ;
    ACanvas.FillRectangle(ABrush,MakeRect(0,0,15,15));
    ABrush.free;
    APen := TGPPen.Create(MakeColor(128,$99,$99,$99),1);
    ACanvas.DrawRectangle(APen,MakeRect(0,0,15,15));
    APen.free;
  ACanvas.Free;

  GPCanvas  := TGPGraphics.Create(canvas.Handle);
  GPCanvas.DrawImage(AnImage,ARect);
  GPCanvas.Free;
  AnImage.Free;
end;

procedure TGDIPDropDownColorSelector.DoDroppedDown;
begin
  inherited;

end;

function TGDIPDropDownColorSelector.DoPopulatePanel(
  APanel: TTntExDropDownPanel): Boolean;
begin
  result := true;
  FColorSelector.parent := APanel;
  FColorSelector.Top := 0;
  FColorSelector.left := 0;
  PanelWidth := FColorSelector.Width + 4;
  PanelHeight := FColorSelector.height + 4;
  FColorSelector.Align := alClient;
end;

function TGDIPDropDownColorSelector.GetAllowAlpha: boolean;
begin
  result := FColorSelector.AlphaEnabled;
end;

function TGDIPDropDownColorSelector.GetAlphaColor: TGPColor;
begin
  result := FColorSelector.AlphaColor;
end;

procedure TGDIPDropDownColorSelector.SetAllowAlpha(const Value: boolean);
begin
  FColorSelector.AlphaEnabled := value;
end;

procedure TGDIPDropDownColorSelector.SetAlphaColor(const Value: TGPColor);
begin
  FColorSelector.AlphaColor := value;
end;

procedure TGDIPDropDownColorSelector.UpdateMargins;
begin
  inherited;
  Perform(EM_SETMARGINS,EC_LEFTMARGIN,(ColorSwashWidth+4));
end;


end.

