{*****************************************************************************}
{                                                                             }
{    GDI + Controls                                                           }
{      http://lummie.co.uk                                                    }
{        Version: 1.0.3                                                       }
{                                                                             }
{    Copyright (c) 2005 Matt harrison (http://www.lummie.co.uk)               }
{                                                                             }
{*****************************************************************************}
unit GDIPImageBrowser;

interface

uses
  windows, SysUtils, Classes, TntExDropDownVirtualDrawTree, GDIPAPI, GDIPOBJ, GDIPUTIL, VirtualTrees, graphics;

type
  TRecType = (rtDir, rtFile);

  TNodeRec = record
    RecType : TRecType;
    FileName : widestring;
    Image : TGPBitmap;
    width : integer;
    height : integer;
  end;
  pNodeRec = ^TnodeRec;


  TImageBackground = (ibNone, ibCheck, ibBlack, ibWhite, ibRed, ibGreen, ibBlue);

  TGDIPImageBrowser = class(TCustomTntExDropDownVirtualDrawTree)
  private
    { Private declarations }
    FRootDir: widestring;
    FFileTypes: widestring;
    FGraphics : TGPGraphics;
    FCheckImage : TGPBitmap;
    FChecksBrush : TGPBrush;
    FImageBackground: TImageBackground;
    FFilename : widestring;
  protected
    { Protected declarations }
    procedure SetImageBackground(const Value: TImageBackground);
    function getFilename: widestring;
    procedure SetFilename(const Value: widestring);
    procedure SetRootDir(const Value: widestring);
    procedure SetFileTypes(const Value: widestring);
    function Build(parentnode: pVirtualNode; Dir: string) : integer;
    function ValidFile(filename: string): boolean;
    function CreateCheckImage: TGPBitmap;
    procedure PaintNode(Sender: TBaseVirtualTree; Node: PVirtualNode; Canvas: TCanvas;
      ContentRect, CellRect: TRect; Column : integer; NodeWidth : integer; IsFocused : boolean);
    procedure FocusChanging(Sender: TBaseVirtualTree; OldNode,
      NewNode: PVirtualNode; OldColumn, NewColumn: TColumnIndex;
      var Allowed: Boolean);
    procedure InitChildren(Sender: TBaseVirtualTree; Node: PVirtualNode; var ChildCount: Cardinal);
    procedure DrawNode(Sender: TBaseVirtualTree; const PaintInfo: TVTPaintInfo);
    procedure FreeNode(Sender: TBaseVirtualTree; Node: PVirtualNode);
    procedure GetNodeWidth(Sender: TBaseVirtualTree; HintCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex; var NodeWidth: Integer);
    procedure Paint; override;
    procedure CleanUpImages;
    procedure CleanUpImagesIterator(Sender: TBaseVirtualTree; Node: PVirtualNode; Data: Pointer; var Abort: Boolean);
    procedure CloseUp; override;
    procedure GetHintSize(Sender: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex; var R: TRect);
    procedure DrawHint(Sender: TBaseVirtualTree; Canvas: TCanvas; Node: PVirtualNode; R: TRect; Column: TColumnIndex);
    procedure Loaded; override;
    procedure SetSelectedIterator(Sender: TBaseVirtualTree; Node: PVirtualNode; Data: Pointer; var Abort: Boolean);
    procedure TreeDoubleClick(Sender: TObject);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure Refresh;
    destructor Destroy; override;
    property Filename : widestring read getFilename write SetFilename;

  published
    { Published declarations }
    property RootDir : widestring read FRootDir write SetRootDir;
    property FileTypes : widestring read FFileTypes write SetFileTypes;
    property ImageBackground : TImageBackground read FImageBackground write SetImageBackground;
    property OnChange;

    property Align;
    property Anchors;
    property AutoSize;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BiDiMode;
    property BorderWidth;
    property Caption;
    property Color;
    property Constraints;
    property Ctl3D;
    property UseDockManager default True;
    property DockSite;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property ParentBiDiMode;
    {$IFDEF COMPILER_7_UP}
    property ParentBackground;
    {$ENDIF}
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnCanResize;
    property OnClick;
    property OnConstrainedResize;
    property OnContextPopup;
    property OnDockDrop;
    property OnDockOver;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnGetSiteInfo;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDock;
    property OnStartDrag;
    property OnUnDock;
  end;

implementation


uses
  GDIPExtensions, Types, Forms, Controls, TntExDropDownControl;

{ TTntExImageBrowser }

function TGDIPImageBrowser.Build(parentnode : pVirtualNode; Dir : string) : integer;
var
  node : pVirtualNode;
  rec : pNodeRec;
  sr : TSearchRec;
  img : TGPImage;
  nheight : integer;
  filename : widestring;
begin
  Screen.Cursor := crHourglass;
  try
    result := 0;
    // process dirs first
    if FindFirst(Dir + '*.*', faAnyFile, sr) = 0 then
    begin
      repeat
        if (sr.Attr and faDirectory = faDirectory) and (sr.Name <> '.') and (sr.Name <> '..') then // ignore . and ..
        begin
          node := tree.AddChild(parentNode);
          node.States := node.states + [vsHasChildren];
          rec := tree.GetNodeData(node);
          rec.RecType := rtDir;
          rec.FileName := IncludeTrailingPathDelimiter(Dir + sr.Name);
          inc(result);
        end;
      until (FindNext(sr) <> 0);
      FindClose(sr);
    end;
    // process files first
    if FindFirst(Dir + '*.*', faAnyFile, sr) = 0 then
    begin
      repeat
        if (sr.Attr and faDirectory <> faDirectory) and (sr.Name <> '.') and (sr.Name <> '..') then // ignore . and ..
        begin
          if ValidFile(sr.name) then
          begin
            filename := Dir + sr.Name;
            // check image can be opened and get height
            try
              img := TGPImage.create(FileName);
              if assigned(img) then
              begin
                node := tree.AddChild(parentNode);


                rec := tree.GetNodeData(node);
                rec.RecType := rtFile;
                rec.FileName := filename;
                rec.width := img.GetWidth;
                rec.height := img.GetHeight;

                nheight := rec.height;
                if nheight < 18 then nheight := 18;
                if nheight > 100 then nheight := 100;
                tree.NodeHeight[node] := nheight;

                inc(result);
              end;
            finally
              FreeAndNil(img);
            end;
          end;
        end;
      until (FindNext(sr) <> 0);
      FindClose(sr);
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TGDIPImageBrowser.Refresh;
begin
  if assigned(tree) then
  begin
    Tree.clear;
    Build(nil,IncludeTrailingPathDelimiter(FRootDir));
  end;
end;

constructor TGDIPImageBrowser.Create(AOwner: TComponent);
var
  tc : TVirtualTreeColumn;
begin
  inherited;
  FImageBackground := ibNone;
  FChecksBrush := nil;
  FCheckImage := nil;
  Tree.NodeDataSize := sizeof(TNodeRec);
  FRootDir := '';
  FGraphics := nil;
  FFileTypes := '.jpg;.jpeg;.png;.bmp;.tif;.gif';
  Tree.OnInitChildren := InitChildren;
  Tree.OnDrawNode := DrawNode;
  Tree.OnGetNodeWidth := GetNodeWidth;
  Tree.OnFreeNode := FreeNode;
  Tree.OnFocusChanging := FocusChanging;
  Tree.OnGetHintSize := GetHintSize;
  Tree.OnDrawHint := DrawHint;
  Tree.OnDblClick := TreeDoubleClick;

  Tree.TreeOptions.MiscOptions := Tree.TreeOptions.MiscOptions + [toVariableNodeHeight];
  Tree.TreeOptions.paintOptions := Tree.TreeOptions.paintOptions + [toShowBackground];
  Tree.Header.Columns.Add;
  tc := Tree.Header.Columns.Add;
  tc.Width := 0;
  tc.MinWidth := 0;
  tc.Options := [coVisible,coParentColor,coParentBidiMode];
  tree.Header.AutoSizeIndex := 0;
  tree.Header.Options := tree.Header.Options + [hoAutoResize];
  Color := clWindow;
  ControlStyle := ControlStyle -[csParentBackground] + [csOpaque];
end;

procedure TGDIPImageBrowser.SetFileTypes(const Value: widestring);
begin
  FFileTypes := Value;
end;

procedure TGDIPImageBrowser.SetRootDir(const Value: widestring);
begin
  FRootDir := Value;
end;

function TGDIPImageBrowser.ValidFile(filename: string): boolean;
var
  fext : widestring;
begin
  fext := ExtractFileExt(filename);
  result := pos(LowerCase(fext)+';',FFileTypes) > 0;
end;

procedure TGDIPImageBrowser.InitChildren(Sender: TBaseVirtualTree;
  Node: PVirtualNode; var ChildCount: Cardinal);
var
  rec : pNodeRec;
begin
  sender.BeginUpdate;
  try
    if not assigned(node) then exit;
    rec := sender.GetNodeData(node);
    if rec.RecType = rtDir then
    begin
      ChildCount := Build(node,rec.FileName);
    end
    else
    begin
      ChildCount := 0;
    end;
  finally
    sender.EndUpdate;
  end;
end;

procedure TGDIPImageBrowser.PaintNode(Sender: TBaseVirtualTree; Node : PVirtualNode; Canvas : TCanvas; ContentRect, CellRect : TRect; Column : integer; NodeWidth : integer;IsFocused : boolean);
var
  rec : pNodeRec;
  ARect : TGPRect;
  x : double;
  img : TGPBitmap;
  w,h : integer;
  destrect : TGPRect;
  r : TRect;
  s : widestring;
  StringSize : TSize;
  IB : TImageBackground;
  tempImage : TGPBitmap;
begin
  if not assigned(FGraphics) or (FGraphics.GetHDC <> Canvas.Handle) then
  begin
    FreeAndNil(FGraphics);
    FGraphics := TGPGraphics.Create(Canvas.Handle);
  end;

  if IsFocused then
    Canvas.Font.Color := clHighlightText
  else
    Canvas.Font.Color := clWindowText;



  rec := sender.GetNodeData(node);
  case rec.RecType of
    rtDir : begin
      // paint text
      R := ContentRect;
      InflateRect(R, -2, 0);
      Dec(R.Right);
      Dec(R.Bottom);


      s := copy(rec.filename,length(FRootDir) + 1,length(rec.filename) - length(FRootDir)-1);
      GetTextExtentPoint32W(canvas.Handle, PWideChar(S), Length(s), StringSize);
      if StringSize.cx > (r.Right-r.Left) then
        S := ShortenString(Canvas.Handle, S, r.Right - r.Left, False);
      SetBKMode(Canvas.Handle, TRANSPARENT);
      DrawTextW(Canvas.Handle, PWideChar(S), Length(S), R, DT_TOP or DT_LEFT or DT_VCENTER or DT_SINGLELINE, False);
    end;

    rtFile : begin
      // paint image
      if not assigned(rec.Image) then
      begin
        screen.Cursor := crHourglass;
        tempImage := TGPBitmap.create(rec.FileName);
        rec.width := tempImage.getWidth;
        rec.height := tempImage.GetHeight;
        rec.Image := tempImage.Clone(0,0,rec.width,rec.height,PixelFormat32bppARGB);
        tempImage.free;
        screen.Cursor := crDefault;
      end;

      if assigned(rec.image) then
      begin
        img := rec.image;
        w := rec.Width;
        h := rec.Height;
        destrect := makerect(ContentRect);
        ARect := MaintainAspect(makerect(0,0,w,h),destrect,x,[roSourceIsMax, roCenterVertical]);

        IB := FImageBackground;

        case IB of
          ibCheck : begin
            if assigned(FCheckImage) then
              FGraphics.FillRectangle(FChecksBrush,ARect)
          end;
          ibBlack : begin
            canvas.Brush.color := clBlack;
            Canvas.FillRect(Rect(ARect.x,ARect.y,ARect.x + ARect.Width,ARect.y + ARect.Height));
          end;
          ibWhite : begin
            canvas.Brush.color := clWhite;
            Canvas.FillRect(Rect(ARect.x,ARect.y,ARect.x + ARect.Width,ARect.y + ARect.Height));
          end;
          ibRed : begin
            canvas.Brush.color := clRed;
            Canvas.FillRect(Rect(ARect.x,ARect.y,ARect.x + ARect.Width,ARect.y + ARect.Height));
          end;
          ibGreen : begin
            canvas.Brush.color := clGreen;
            Canvas.FillRect(Rect(ARect.x,ARect.y,ARect.x + ARect.Width,ARect.y + ARect.Height));
          end;
          ibBlue : begin
            canvas.Brush.color := clBlue;
            Canvas.FillRect(Rect(ARect.x,ARect.y,ARect.x + ARect.Width,ARect.y + ARect.Height));
          end;
        end;
        screen.Cursor := crHourglass;
        FGraphics.DrawImage(img,ARect);
        screen.Cursor := crDefault;

        // paint text
        R := ContentRect;
        r.left := ARect.X + ARect.width;
        InflateRect(R, -2, 0);
        Dec(R.Right);
        Dec(R.Bottom);
        s := ExtractFileName(rec.filename);

        GetTextExtentPoint32W(canvas.Handle, PWideChar(S), Length(s), StringSize);
        if StringSize.cx > (r.Right-r.Left) then
          S := ShortenString(Canvas.Handle, S, r.Right - r.Left, False);
        SetBKMode(Canvas.Handle, TRANSPARENT);
        DrawTextW(Canvas.Handle, PWideChar(S), Length(S), R, DT_TOP or DT_LEFT or DT_VCENTER or DT_SINGLELINE, False);
      end;

    end;
  end;
end;

procedure TGDIPImageBrowser.DrawNode(Sender: TBaseVirtualTree;
  const PaintInfo: TVTPaintInfo);
begin
  if not assigned(PaintInfo.node) then
  begin
    exit;
  end;

  if not assigned(FCheckImage) then FCheckImage := CreateCheckImage;

  with PaintInfo do
  begin
    PaintNode(Sender, Node, Canvas, ContentRect, CellRect, Column, NodeWidth, (vsSelected in Node.states));
  end;
end;

destructor TGDIPImageBrowser.Destroy;
begin
  FreeAndNil(FGraphics);
  FreeAndNil(FChecksBrush);
  FreeAndNil(FCheckImage);
  inherited;
end;

procedure TGDIPImageBrowser.FreeNode(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
var
  rec : pNoderec;
begin
  if assigned(node) then
  begin
    rec := Sender.GetNodeData(node);
    if assigned(rec) then
    begin
      FreeAndNil(rec.Image);
    end;
  end;
end;

function TGDIPImageBrowser.CreateCheckImage : TGPBitmap;
var
  ACanvas : TGPGraphics;
  ABrush : TGPSolidBrush;
begin
  try
    result := TGPBitmap.Create(7,7,PixelFormat24bppRGB);
    ACanvas := TGPGraphics.Create(result);
    ABrush := TGPSolidBrush.Create(MakeColor(255,255,255));
    ACanvas.FillRectangle(ABrush,MakeRect(0,0,7,7));
    ABrush.free;
    ABrush := TGPSolidBrush.Create(MakeColor(192,192,192));
    ACanvas.FillRectangle(ABrush,MakeRect(0,0,3,3));
    ACanvas.FillRectangle(ABrush,MakeRect(4,4,7,7));
    ABrush.free;
    ACanvas.Free;
//    FChecksBrush := TGPSolidBrush.create(makecolor(0,0,$99));
    FChecksBrush := TGPTextureBrush.create(result,WrapModeTileFlipXY);
  except
    result := nil;
  end;
end;


procedure TGDIPImageBrowser.GetNodeWidth(Sender: TBaseVirtualTree;
  HintCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  var NodeWidth: Integer);
var
  rec : pNodeRec;
begin
  if column = 0 then
  begin
    rec := sender.GetNodeData(node);
    case rec.RecType of
      rtDir, rtFile : NodeWidth := (sender as TVirtualDrawTree).Header.Columns[Column].Width;
    end;
  end;
end;

procedure TGDIPImageBrowser.Paint;
var
  r : TRect;
begin
  inherited;
  if assigned(Tree.FocusedNode) then
  begin
    r := ClientRect;
    inflaterect(r,0,-6);
    r.Right := r.Right - GetButtonWidth;
    r.left := r.left + 6;
    PaintNode(tree,Tree.FocusedNode, Canvas, r,r,0,ClientWidth, Focused);
  end;
end;

procedure TGDIPImageBrowser.FocusChanging(Sender: TBaseVirtualTree;
  OldNode, NewNode: PVirtualNode; OldColumn, NewColumn: TColumnIndex;
  var Allowed: Boolean);
var
  rec : pNodeRec;
begin
  if assigned(newNode) then
  begin
    rec := sender.GetNodeData(newnode);
    case rec.RecType of
      rtDir : Allowed := true;
    else
      Allowed := true;
    end;
  end;
end;



procedure TGDIPImageBrowser.CleanUpImages;
begin
  Tree.IterateSubtree(nil,CleanUpImagesIterator,nil,[vsInitialized],false);
end;

procedure TGDIPImageBrowser.CleanUpImagesIterator(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Data: Pointer; var Abort: Boolean);
var
  rec : pNodeRec;
begin
  if assigned(node) and (node <> sender.FocusedNode) then
  begin
    rec := sender.GetNodeData(node);
    if assigned(rec.Image) then FreeAndNil(rec.image);
  end;
end;

procedure TGDIPImageBrowser.CloseUp;
begin
  inherited;
  CleanUpImages;
end;

procedure TGDIPImageBrowser.GetHintSize(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; var R: TRect);
var
  p : TPoint;
  DeskRect : TRect;
  ImageRect : TRect;
  rec : pNodeRec;
  a : TGPRect;
  x : double;
begin
  if assigned(node) then
  begin
    GetCursorPos(p);
    DeskRect := Screen.DesktopRect;
    rec := sender.GetNodeData(node);
    if rec.RecType = rtFile then
    begin
      ImageRect := Rect(p.x, p.y, p.x + rec.width, p.Y + rec.height);
      IntersectRect(r,ImageRect,DeskRect);
      OffsetRect(r,-r.Left,-r.top);
      a := MaintainAspect(MakeRect(0,0,rec.width,rec.height),makerect(r),x,[roSourceIsMax]);
      r.Right := a.Width;
      r.Bottom := a.Height;
    end;
  end;
end;

procedure TGDIPImageBrowser.DrawHint(Sender: TBaseVirtualTree;
  Canvas: TCanvas; Node: PVirtualNode; R: TRect; Column: TColumnIndex);
var
  rec : pNoderec;
begin
  if assigned(node) and (column=0)then
  begin
    rec := sender.GetNodeData(node);
    if (rec.RecType = rtFile) and assigned(rec.image) then
    begin
      BitBlt(canvas.Handle,0,0,r.Right,r.Bottom,0,0,0,WHITENESS);
      with TGPGraphics.create(canvas.Handle) do
      begin
        DrawImage(rec.image, makerect(r));
        free;
      end;
    end;
  end;
end;

procedure TGDIPImageBrowser.Loaded;
begin
  inherited;
  Tree.PopupMenu := PopupMenu;
  if ShowHint then
  begin
    Tree.ShowHint := true;
    Tree.HintMode := hmHint;
    Tree.HintAnimation := hatSlide;
  end;
end;

procedure TGDIPImageBrowser.SetImageBackground(
  const Value: TImageBackground);
begin
  FImageBackground := Value;
  Tree.Invalidate;
end;

function TGDIPImageBrowser.getFilename: widestring;
var
  rec : pNodeRec;
begin
  result := '';
  if assigned(tree.FocusedNode) then
  begin
    rec := Tree.GetNodeData(Tree.FocusedNode);
    if rec.RecType = rtFile then
    begin
      result := rec.FileName;
    end;
  end;
end;

procedure TGDIPImageBrowser.SetFilename(const Value: widestring);
var
  node: PVirtualNode;
begin
  FFilename := value;
  Tree.ClearSelection;
  Tree.FocusedNode := nil;
  node := Tree.IterateSubtree(nil,SetSelectedIterator,nil,[],True,false);
  if assigned(node) then
  begin
    tree.Selected[node] := true;
    tree.FocusedNode := Node;
  end;
  paint;
end;

procedure TGDIPImageBrowser.SetSelectedIterator(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Data: Pointer; var Abort: Boolean);
var
  rec : pNodeRec;
begin
  if assigned(node) then
  begin
    rec := sender.GetNodeData(node);
    if assigned(rec) then
    begin
      if WideSameText(rec.FileName,FFilename) then
      begin
        Abort := true;
      end;
    end;
  end;
end;

procedure TGDIPImageBrowser.TreeDoubleClick(Sender: TObject);
begin
  CloseUp;
end;

end.
